@@ -338,9 +338,7 @@ impl <A, T> RangeSet <A> where
             std::cmp::max (*range.end(), *self.ranges[after-1].end());
           // if there are more than one ranges between we must shift and truncate
           if 1 < after - before - 1 {
-            for i in 0..(after-before-1) {
-              self.ranges[before+1+i] = self.ranges[after + i].clone();
-            }
+            self.ranges.as_mut_slice()[(before + 2)..].rotate_left(after - before - 2);
             let new_len = self.ranges.len() - (after - before - 2);
             self.ranges.truncate (new_len);
           }
@@ -638,6 +636,16 @@ impl <A, T> From <std::ops::RangeInclusive <T>> for RangeSet <A> where
   }
 }
 
+impl <A, T> AsRef <smallvec::SmallVec <A>> for RangeSet <A> where
+  A : smallvec::Array <Item=std::ops::RangeInclusive <T>>
+    + Eq + std::fmt::Debug,
+  T : PrimInt + std::fmt::Debug
+{
+  fn as_ref (&self) -> &smallvec::SmallVec <A> {
+    &self.ranges
+  }
+}
+
 impl <'a, A, T> Iterator for Iter <'a, A, T> where
   A : smallvec::Array <Item=std::ops::RangeInclusive <T>>
     + Eq + std::fmt::Debug,
@@ -673,7 +681,15 @@ mod tests {
     range_set.insert_range(3..=3);
     range_set.insert_range(5..=5);
     range_set.insert_range(7..=7);
-    range_set.insert_range(1..=9);
+    assert_eq!(
+      range_set.insert_range(1..=9),
+      {
+        let mut r = RangeSet::from(3..=3);
+        r.insert_range(5..=5);
+        r.insert_range(7..=7);
+        Some(r)
+      }
+    );
 
     assert_eq!(range_set.ranges.into_vec(), vec!(1..=9));
   }
@@ -684,7 +700,14 @@ mod tests {
     range_set.insert_range(3..=3);
     range_set.insert_range(5..=5);
     range_set.insert_range(9..=9);
-    range_set.insert_range(1..=7);
+    assert_eq!(
+      range_set.insert_range(1..=7),
+      {
+        let mut r = RangeSet::from(3..=3);
+        r.insert_range(5..=5);
+        Some(r)
+      }
+    );
 
     assert_eq!(range_set.ranges.into_vec(), vec!(1..=7, 9..=9));
   }
@@ -695,8 +718,36 @@ mod tests {
     range_set.insert_range(1..=1);
     range_set.insert_range(5..=5);
     range_set.insert_range(7..=7);
-    range_set.insert_range(3..=9);
+    assert_eq!(
+      range_set.insert_range(3..=9),
+      {
+        let mut r = RangeSet::from(5..=5);
+        r.insert_range(7..=7);
+        Some(r)
+      }
+    );
 
     assert_eq!(range_set.ranges.into_vec(), vec!(1..=1, 3..=9));
   }
+
+  #[test]
+  fn gap_then_merge_multiple_then_gap() {
+    let mut range_set: RangeSet<[RangeInclusive<u32>; 2]> = RangeSet::new();
+    range_set.insert_range(1..=1);
+    range_set.insert_range(3..=3);
+    range_set.insert_range(5..=5);
+    range_set.insert_range(7..=7);
+    range_set.insert_range(9..=9);
+    assert_eq!(
+      range_set.insert_range(3..=7),
+      {
+        let mut r = RangeSet::from(3..=3);
+        r.insert_range(5..=5);
+        r.insert_range(7..=7);
+        Some(r)
+      }
+    );
+
+    assert_eq!(range_set.ranges.into_vec(), vec!(1..=1, 3..=7, 9..=9));
+  }
 }
